#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys

if sys.version_info >= (3, 0):
    import configparser
else:
    # Py2K
    import ConfigParser as configparser
    # end Py2K

import os
import unittest

from .client import (Client, Customer,
    CreditCard, Order, korta_reference)


_path = lambda *x: os.path.join(os.path.abspath(
        os.path.dirname(__file__)), *x)


config = configparser.RawConfigParser(allow_no_value=True)
config.readfp(open('userconfig.ini'))


class KortaTestCase(unittest.TestCase):

    def setUp(self):

        self.client = Client(
            _path(config.get('korta', 'pem_path')),
            _path(config.get('korta', 'ca_path')),
            config.get('korta', 'user'),
            config.get('korta', 'password'),
            config.get('korta', 'site_id'),
            config.get('korta', 'card_acceptor_id'),
            config.get('korta', 'card_acceptor_identity'),
            config.get('korta', 'host'),
        )

    def test_charge_reference(self):
        unique_reference = korta_reference()
        self.client.save_account(Customer(
            unique_reference,
            CreditCard(
                '4571999400007492',
                5, 14, 123),
        ))
        order_ref = korta_reference()
        auth = self.client.request_authorization(
            Order(order_ref, 1000),
            reference=unique_reference)
        cap = self.client.request_capture(auth)
        self.assertEqual(cap.action_code, '000')

    def test_one_off(self):
        x = korta_reference()
        c = CreditCard('4571999400007492', 5, 14, 123)
        o = Order(x, 2000)
        self.assertEqual(self.client.one_off(o, c), True)



#!/usr/bin/env python
# -*- coding: utf-8 -

import sys

if sys.version_info >= (3, 0):
    from io import StringIO
else:
    try:
        from cStringIO import StringIO
    except ImportError:
        from StringIO import StringIO

import logging
import pycurl


class SSLClient(object):
    """
    Curl client for SSL certificates & CA bundles.
    """

    def __init__(self, host, port=443):
        self.host = host
        self.port = port
        self.log = logging.getLogger(__name__)
        self.log.debug('SSLClient init host:%s, port:%d' % (host, port))

    def set_cert(self, key_file=None, ca_info=None, passwd_cb=None):
        self.key_file = key_file
        self.ca_info = ca_info
        self.ssl_pass = passwd_cb
        self.log.debug('Certificate set pem:%s, ca:%s, password callback:%s' % (
            key_file, ca_info, passwd_cb))

    def set_user(self, httpuser, httppwd):
        """
        Sets the curl --user option httpuser:httppwd
        """
        self.log.debug('setting user:pass as %s:%s' % (httpuser, httppwd))
        self.user = httpuser
        self.pwd = httppwd

    def request(self, method, url, postdata=None, timeout=None):
        """
        Does a curl post request, optional timeout.

        No exception handling whatsoever, the reason being that SSL
        can be somewhat fickle and therefore the code using this client
        should handle those intricacies
        """

        if not url.startswith('http'):
            url = 'https://%s:%s%s' % (self.host, self.port, url)
        self.log.debug(url)

        buffer = StringIO()
        curl = pycurl.Curl()

        def _debug(debug_type, debug_msg):
            self.log.debug("debug(%d): %s" % (debug_type, debug_msg))

        curl.setopt(pycurl.VERBOSE, 0)
        curl.setopt(pycurl.DEBUGFUNCTION, _debug)

        if method == 'POST' and postdata:
            curl.setopt(pycurl.POST, 1)
            curl.setopt(pycurl.POSTFIELDS, postdata)

        if timeout:
            curl.setopt(pycurl.TIMEOUT_MS, timeout)

        curl.setopt(pycurl.URL, url)
        curl.setopt(pycurl.WRITEFUNCTION, buffer.write)
        curl.setopt(pycurl.NOSIGNAL, 1)

        curl.setopt(pycurl.SSLVERSION, 3)
        self.log.debug('certfile: %s' % self.key_file)
        curl.setopt(pycurl.SSLCERT, self.key_file)

        curl.setopt(pycurl.SSLCERTPASSWD, self.ssl_pass)

        curl.setopt(pycurl.SSL_VERIFYPEER, 0)
        curl.setopt(pycurl.SSL_VERIFYHOST, 2)

        if self.ca_info:
            self.log.debug('CAINFO: %s' % self.ca_info)
            curl.setopt(pycurl.CAINFO, self.ca_info)

        curl.setopt(pycurl.USERPWD,
            '%s:%s' % (self.user, self.pwd))

        self.log.debug('curl.USERPWD: %s:%s' % (self.user, self.pwd))

        curl.perform()
        buffer.seek(0)
        r = buffer.getvalue()
        buffer.close()
        return r



#!/usr/bin/env python
# -*- coding: utf-8 -


class AttributeDictMixin(object):
    """Adds attribute access to mappings.

    >>> d.key -> d[key]
    """

    def __getattr__(self, key):
        """`d.key -> d[key]`"""
        try:
            return self[key]
        except KeyError:
            raise AttributeError("'%s' object has no attribute '%s'" % (
                    self.__class__.__name__, key))

    def __setattr__(self, key, value):
        """`d[key] = value -> d.key = value`"""
        self[key] = value


class AttributeDict(dict, AttributeDictMixin):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -
"""
Values are mapped back and forth between the client and the service,
this module contains the mappings.
"""

from .datastructures import AttributeDict


KORTA = AttributeDict({
    'acceptor_ident': 'd42',
    'acceptor_term_id': 'd41',
    'action_code': 'd39',
    'action_code_echo': 'o39',
    'action_code_text': 'd39text',
    'amount': 'd4',
    'amount_echo': 'o4',
    'approval_code': 'd38',
    'approval_code_echo': 'o38',
    'card_acceptor_id': 'd42',
    'card_acceptor_identity': 'd41',
    'card_brand_name': 'd2brand',
    'cardholder_name': 'd2name',
    'cc_expire': 'd14',
    'cc_number': 'd2',
    'cc_verify_code': 'd47',
    'charge_reference': 'daskauthorise',
    'client_ip': 'cip',
    'client_lang': 'clang',
    'code_authorization_approved': '000',
    'code_authorization_declined': '100',
    'code_system_failure_error': '909',
    'code_system_failure_retry': '946',
    'currency_code': 'd49',
    'currency_exponent': 'de4',
    'current_date': 'd12',
    'current_date_echo': 'o12',
    'date_format': '%y%m%d%H%M%S',
    'error_code': 'error',
    'error_text': 'errortext',
    'function_identifier': 'do',
    'korta_reference': 'dask',
    'mask_cc': 'd2dsp',
    'merchant_lang': 'mlang',
    'original_data_element': 'd56',
    'password': 'pwd',
    'reference': 'daskm',
    'reference_id': 'd31',
    'settlement_reference_number': 'd37',
    'shipping_address': 'd2saddr',
    'shipping_city': 'd2scity',
    'shipping_country': 'd2sctr',
    'shipping_zip': 'd2szip',
    'site': 'site',
    'storage_duration': 'daskd',
    'sub_function_identifier': 'dos',
    'total_response': 'totalResponse',
    'unknown': 'd3',
    'user': 'user',
})


CURRENCY_CODES = AttributeDict({
    'DDK': 208,
    'EUR': 978,
    'GBP': 826,
    'ISK': 352,
    'NOK': 578,
    'SEK': 752,
    'USD': 840,
})



#!/usr/bin/env python
# -*- coding: utf-8 -

import datetime
import itertools
import logging
import random
import string
import sys

if sys.version_info >= (3, 0):
    from urllib.parse import unquote_plus, urlencode
else:
    from urllib import unquote_plus, urlencode

from .datastructures import AttributeDict
from .defaults import KORTA, CURRENCY_CODES
from .sslclient import SSLClient


RKORTA = dict(((v, k) for (k, v)
    in list(KORTA.items())))


RCURRENCY_CODES = dict(((v, k) for (k, v)
    in list(CURRENCY_CODES.items())))


def korta_reference():
    return ''.join([random.choice(string.letters)
        for i in range(19)])


class CreditCard(object):
    """
    Simple data class representing a credit card.

    .. note::

        Will be moved into a separate module
    """

    def __init__(self, number, expiration_month, expiration_year, ccv):
        self.number = number
        self.expiration_month = expiration_month
        self.expiration_year = expiration_year
        self.ccv = ccv

    @property
    def expiration_date(self):
        y = str(self.expiration_year)
        m = self.expiration_month
        if len(y) == 2:
            y = '20' + y
        return datetime.date(int(y), int(m), 1)

    @property
    def expires(self):
        return self.expiration_date.strftime('%y%m')


class Customer(object):
    """
    Simple data class representing a customer

    .. note::

        Will be moved into a separate module
    """

    def __init__(self, reference, credit_card, duration=2):
        self.credit_card = credit_card
        self.reference = reference
        self.duration = duration


class Order(object):
    """
    Simple data class representing an order

    .. note::

        Will be moved into a separate module
    """

    def __init__(self, reference, amount, currency='ISK',
            currency_exponent=2):
        self.reference = reference
        self.amount = amount
        self._currency = currency
        self.currency_exponent = currency_exponent

    @property
    def currency(self):
        return CURRENCY_CODES[self._currency]


class Client(object):
    """
    Encapsulates the logic needed to interface with korta through
    an instance of the SSLClient.
    """

    def __init__(self, key_file, ca_file, user, password, site_id,
            card_acceptor_id, card_acceptor_identity,
            host, port=8443, currency='USD'):

        self.connection = SSLClient(host, port)
        self.connection.set_cert(key_file, ca_file,
            passwd_cb=password)
        self.connection.set_user(user, password)

        self.user = user
        self.password = password
        self.site_id = site_id
        self.card_acceptor_id = card_acceptor_id
        self.card_acceptor_identity = card_acceptor_identity
        self.default_currency = currency
        self.log = logging.getLogger(__name__)

    def parse_response(self, response):
        """
        Maps back to korta values and returns the dictionary
        """
        def _format(k, v):
            return (RKORTA.get(k), v)

        return AttributeDict([_format(k, unquote_plus(v))
                for (k, v) in itertools.chain([i.split('=')
                    for i in response.split('&')])])

    def do_request(self, path, params):
        """
        Calls the path with the urlencoded params
        """
        self.log.debug('Calling %s, params: %s' % (path, repr(params)))
        r = self.parse_response(
            self.connection.request('GET', '%s?%s' % (path,
                urlencode(params))))
        if r.action_code != '000':
            self.log.error('Error: %s' % r.error_text)
        return r

    def get_defaults(self):
        """
        Returns defaults needed for most requests
        """
        return  {
            KORTA.user: self.user,
            KORTA.password: self.password,
            KORTA.site: self.site_id,
            KORTA.card_acceptor_id: self.card_acceptor_id,
            KORTA.card_acceptor_identity: self.card_acceptor_identity
        }

    def get_default_currency(self):
        """
        Only used for account registration
        """
        return CURRENCY_CODES.get(self.default_currency)

    def account_action(self, customer, add=True, path='/rpc/RequestData'):
        """
        Performs either an add or delete action for a korta account
        """
        params = self.get_defaults()
        params.update({
            KORTA.function_identifier: 1,
            KORTA.sub_function_identifier: 'A' if add else 'D',
            KORTA.reference: customer.reference,
        })
        if add:
            params.update({
                KORTA.charge_reference: 1,
                KORTA.amount: 100,
                KORTA.currency_exponent: 2,
                KORTA.currency_code: self.get_default_currency(),
                KORTA.storage_duration: customer.duration,
                KORTA.cc_number: customer.credit_card.number,
                KORTA.cc_verify_code: customer.credit_card.ccv,
                KORTA.cc_expire: customer.credit_card.expires})

        return self.do_request(path, params)

    def request_authorization(self, order, dt=None, reference=None, cc=None,
            ccv=None, cc_expire=None, path='/rpc/RequestAuthorisation'):
        """
        Requests authorization for a transaction
        """
        dt = (dt or datetime.datetime.now()).strftime(
            KORTA.date_format)

        params = self.get_defaults()
        params.update({
            KORTA.amount: order.amount,
            KORTA.currency_code: order.currency,
            KORTA.currency_exponent: order.currency_exponent,
            KORTA.current_date: dt,
            KORTA.reference_id: order.reference,
        })

        if reference:
            params.update({
                KORTA.reference: reference,
                KORTA.charge_reference: 1})

        if all((cc, cc_expire)):
            params.update({
                KORTA.cc_number: cc,
                KORTA.cc_expire: cc_expire})
        if ccv:
            params.update({KORTA.cc_verify_code : ccv})

        return self.do_request(path, params)

    def one_off_req(self, order, credit_card, dt=None):
        return self.request_authorization(order, cc=credit_card.number,
            ccv=credit_card.ccv, cc_expire=credit_card.expires, dt=dt)

    def one_off(self, order, credit_card, dt=None):
        """
        Does a one off
        """
        r = self.one_off_req(order, credit_card, dt=dt)
        if r.action_code != '000':
            self.log.error('Response error: %s' % r.error_text)
            return False
        return (self.request_capture(r).action_code == '000')

    def request_capture(self, _params, path='/rpc/RequestCapture'):
        """
        You should only be calling this function with the result dictionary
        of a request_authorization call.
        """

        drop = ['MASK_CC',
            'ACTION_CODE_TEXT',
            'ERROR_TEXT',
            'ERROR_CODE',
            'CARD_BRAND_NAME',
            'UNKNOWN',
        ]

        params = dict([(KORTA.get(k, k), v)
            for (k, v) in list(_params.items())
                if k not in drop])

        for i in ['12', '39', '38']:
            params['o%s' % i] = params['d%s' % i]

        params.update({
            KORTA.user: self.user,
            KORTA.password: self.password})

        return self.do_request(path, params)

    def save_account(self, customer):
        """
        Saves an instance of a customer
        """
        return (self.account_action(customer).action_code == '000')

    def delete_account(self, customer):
        """
        Deletes an instance of a customer

        """
        return (self.account_action(customer,
            add=False).action_code == '000')



#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages


setup(
    name = "korta",
    version = "0.1.3",
    packages = find_packages(),
    install_requires = [
        'pycurl',
    ],
    package_data = {
        'korta': ['certs/*.pem', 'certs/*.crt'],
    },
    author="Stefan Kjartansson",
    author_email="esteban.supreme@gmail.com",
    description="Korta Client library",
    long_description=open('README.rst').read(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
    ],
)
